/*
 * Click nbfs://nbhost/SystemFileSystem/Templates/Licenses/license-default.txt to change this license
 * Click nbfs://nbhost/SystemFileSystem/Templates/Classes/Class.java to edit this template
 */
package com.jcg.cleanerexample;

import java.io.FileWriter;
import java.io.IOException;
import java.lang.ref.Cleaner;

/**
 *
 * @author omozegieaziegbe
 */
public class ResourceHandler implements AutoCloseable {

    private final Cleaner.Cleanable cleanable;
    private final FileWriter fileWriter;

    public ResourceHandler(String fileName) throws IOException {
        this.fileWriter = new FileWriter(fileName);
        this.cleanable = Cleaner.create().register(this, new CleanerRunnable(fileWriter));
    }

    // Implementing close method of AutoCloseable interface
    @Override
    public void close() throws IOException {
        cleanable.clean();
    }

    // Method to write data to the resource
    public void write(String data) throws IOException {
        fileWriter.write(data);
        fileWriter.flush(); // Flush data to ensure it's written immediately
    }

    // Runnable for cleaner
    private static class CleanerRunnable implements Runnable {

        private final FileWriter fileWriter;

        public CleanerRunnable(FileWriter fileWriter) {
            this.fileWriter = fileWriter;
        }

        @Override
        public void run() {
            try {
                fileWriter.close();
                System.out.println("Resource cleaned up by Cleaner API.");
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }
}
